#!/usr/bin/perl 

use strict;
use lib '/opt/tnt/lib';
use tntutil;
use Getopt::Std;
use DBI;

my($do_machine) = 0;
my($do_user) = 0;
my($filename);
my($create_if_missing) = 0;

sub
parseOpts {
	my(%opts);
	getopts('cumf:', \%opts);

	if ($opts{u}) {
		if ($do_machine == 1) {
			die "Cannot import users and machines at one time\n";
		}
		$do_user = 1;
	}

	if ($opts{m}) {
		if ($do_user == 1) {
			die "Cannot import users and machines at one time\n";
		}
		$do_machine = 1;
	}
	if ($opts{c}) {
		$create_if_missing = 1;
	}
	if (!$do_user && !$do_machine) {
		die "Must specified whether to import users (-u) or machines (-m)\n";
	}
	if (!$opts{f})  {
		die "Filename must be specified with -f\n";
	}
	$filename = $opts{f};
	if (! -f $filename) {
		die "File $filename not found or not valid file\n";
	}
}

sub
getDatabaseConnectionLocal {
	my($dbhost) = getValueFromFile("PGHOST", "/etc/tntdb.conf");
	my($dbname) = getValueFromFile("PTBASE", "/etc/tntdb.conf");
	my($dbport) = getValueFromFile("PGPORT", "/etc/tntdb.conf");
	my($dbuser) = getValueFromFile("DBUSER", "/etc/tntdb.conf");
	my($dbpass) = getValueFromFile("DBPASS", "/etc/tntdb.conf");

	my $dbh = getDatabaseConnection($dbname, $dbhost, $dbport, $dbuser, $dbpass, { PrintError => 1,  AutoCommit => 0 });
	if (!$dbh) {
		die "Error getting database connection: $!\n";
	}

	return $dbh;
}

sub
createObject {
	my($dbh, $name) = @_;
	my($sth) = ($do_machine == 1) ?  $dbh->prepare("insert into tnt_config.policynode (name, type, status) values (?, 5, 0)") 
		: $dbh->prepare("insert into tnt_config.policyidentity (name, type, domainid) values (?, 2, (select id from tnt_config.domain where system_object is true))");
	$sth->bind_param(1, $name);
	$sth->execute;
	$sth->finish;
	# get id
	my($id) = verifyObject($dbh, $name, 1);
	$sth = $dbh->prepare("insert into tnt_config." . (($do_machine == 1) ? "nodegroup" : "identitygroup") . " (id) values (?)");
	$sth->bind_param(1, $id);
	$sth->execute;
	$sth->finish;

	return $id;
}

sub
verifyObject {
	my($dbh, $name, $is_group) = @_;
	my($table) = ($do_machine == 1) ? "policynode" : "policyidentity";
	my($member_type) = ($do_machine == 1) ? 2 : 1;
	my($group_type) = ($do_machine == 1) ? 5 : 2;
	my($id);
	# my $ary_ref = $dbh->selectcol_arrayref("select id from tnt_config.$table where name = ? and delete_time is null and type = ?", undef, ( $name, (defined($is_group) ? $group_type : $member_type) ));
	my $ary_ref = $dbh->selectcol_arrayref("select id from tnt_config.$table where name ilike ? and delete_time is null and type = ?", undef, ( $name, (defined($is_group) ? $group_type : $member_type) ));
	# we only auto-create groups
	if (defined($$ary_ref['id'])) {
		$id = $$ary_ref['id'];
	} elsif (($is_group == 1) && ($create_if_missing == 1)) {
		$id = createObject($dbh, $name, $is_group);
	} else {
		# warn "No object found with name $name\n";
		warn "No " . ($do_machine == 1 ? "machine" : "user") . " found with name $name\n";
	}

	return $id; 
}

sub
associate {
	my($dbh, $memberId, $groupId, $memberName, $groupName) = @_;

	my($table) = ($do_machine == 1) ? "nodegroup_node" : "identitygroup_identity";
	my($member_id_col) = ($do_machine == 1) ? "nodeid" : "uid";
	my($group_id_col) = ($do_machine == 1) ? "nodegroupid" : "groupid";
	my $ary_ref = $dbh->selectcol_arrayref("select count(*) from tnt_config.$table where $member_id_col = ? and $group_id_col = ? and delete_time is null", undef, ( $memberId, $groupId )); 
	if ($$ary_ref['count'] == 0) {
		my($sth) = $dbh->prepare("insert into tnt_config.$table ($member_id_col, $group_id_col, creation_time) values (?, ?, now())"); 
		my($rv) = $sth->execute($memberId, $groupId);
		if ($rv != 1) {
			warn("Error associating $memberName with group $groupName: $!\n");
		}
	}
}

parseOpts();
my($dbh) = getDatabaseConnectionLocal();
eval {
	my($line, $memberId, @group_names, $group_name, $groupId);
	open(FILE, $filename) || die "Error opening file $filename: $!\n";
	foreach $line (<FILE>) {
		chomp($line);
		my($member, $groups) = split(/\,/, $line);
		$memberId = verifyObject($dbh, $member);
		if (defined($memberId)) {
			@group_names = split(/\;/, $groups);
			for $group_name (@group_names) {
				$groupId = verifyObject($dbh, $group_name, 1);
				if (defined($groupId)) {
					associate($dbh, $memberId, $groupId, $member, $group_name);
				}
			}
		}
	}
	close(FILE);
};
if ($@) {
	$dbh->rollback();	
} else {
	$dbh->commit();
}
$dbh->disconnect();
